html,button,input,select,textarea{color:#222}.chromeframe{margin:.2em 0;background:#ccc;color:#000;padding:.2em 0}.unselectable{-moz-user-select:-moz-none;-webkit-user-select:none;-o-user-select:none;-ms-user-select:none;user-select:none}.ir{background-color:transparent;border:0;overflow:hidden;text-indent:-9999px}.ir:before{content:"";display:block;width:0;height:150%}.hidden{display:none!important;visibility:hidden}.visuallyhidden{border:0;clip:rect(0 0 0 0);height:1px;margin:-1px;overflow:hidden;padding:0;position:absolute;width:1px}.visuallyhidden.focusable:active,.visuallyhidden.focusable:focus{clip:auto;height:auto;margin:0;overflow:visible;position:static;width:auto}body{font:13px / 1.5 'Helvetica Neue',Arial,'Liberation Sans',FreeSans,sans-serif;overflow:hidden;height:100vh;background-color:#1d1f21;width:100%}#toolbar{width:100%;background:#282a2e;padding-top:5px;padding-left:5px;padding-right:5px;height:30px;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-webkit-flex-direction:row;-ms-flex-direction:row;flex-direction:row;-webkit-flex-wrap:nowrap;-ms-flex-wrap:nowrap;flex-wrap:nowrap;-webkit-box-pack:start;-webkit-justify-content:flex-start;-ms-flex-pack:start;justify-content:flex-start;-webkit-align-content:stretch;-ms-flex-line-pack:stretch;align-content:stretch;-webkit-box-align:center;-webkit-align-items:center;-ms-flex-align:center;align-items:center}.toolbar-item{height:26px;padding-right:5px;-webkit-box-flex:0;-webkit-flex:0 0 auto;-ms-flex:0 0 auto;flex:0 0 auto;-webkit-box-ordinal-group:1;-webkit-order:0;-ms-flex-order:0;order:0;-webkit-align-self:flex-start;-ms-flex-item-align:start;align-self:flex-start}.toolbar-item-fill{-webkit-box-flex:1;-webkit-flex:1 0 auto;-ms-flex:1 0 auto;flex:1 0 auto}div #file-select{min-width:22%}div #command-select{min-width:80px}#script-input{box-sizing:border-box;box-shadow:2px 2px 2px 0 #202225;height:26px;background:#373b41;padding-right:30px;position:relative;font-size:13px}#script-input input{padding-left:.5em;height:100%;width:100%;text-align:center}#script-input div{line-height:26px;color:#888;text-align:center;vertical-align:middle;cursor:pointer;cursor:hand;display:block;position:absolute;right:0;top:0;width:22px}#script-input div+div{right:22px}.toolbar-widget,#script-input input,.button-group a{color:#c5c8c6;background:#373b41;border:0;border-radius:0;box-sizing:border-box}.toolbar-hover,#script-input div:hover,.button-group a:hover,.button-group a.selected{background-color:#43484f}.toolbar-text,#script-input input{font-family:"Consolas","Menlo","Monaco","Lucida Console","Liberation Mono","DejaVu Sans Mono","Bitstream Vera Sans Mono","Courier New","monospace","serif";font-size:13px}.button-group{height:26px;padding-right:5px}.button-group a{box-shadow:2px 2px 2px 0 #202225;float:left;margin-right:0;height:26px;width:26px;text-align:center;font-size:1.5em}.button-group a:link,.button-group a:visited,.button-group a:active{text-decoration:none;outline:none}#configuration{position:absolute;top:5px;right:15px}#action-show-settings{position:absolute;top:5px;right:15px}#minimized-action-bar{position:absolute;top:3px;right:8px}.scrollable{height:100%;word-break:break-all;white-space:nowrap;overflow:auto}.log-view{font-family:"Consolas","Menlo","Monaco","Lucida Console","Liberation Mono","DejaVu Sans Mono","Bitstream Vera Sans Mono","Courier New","monospace","serif";font-size:13px;line-height:1.3;padding-top:2px;color:#c5c8c6;background-color:#1d1f21}.log-view .log-entry{display:block}.log-view .log-entry-current{background:#282a2e}.log-view .log-notice{background:#FF974F;color:#1d1f21;text-align:center;font-weight:bold}.log-view-wrapped{white-space:normal}.log-view ::-moz-selection{background:#373b41;text-shadow:none}.log-view ::selection{background:#373b41;text-shadow:none}input::-moz-selection{background:black;text-shadow:none;color:white}input::selection{background:black;text-shadow:none;color:white}input::-webkit-input-placeholder{color:#c5c8c6;font-style:italic;text-align:center}input::-moz-placeholder{color:#c5c8c6;font-style:italic;text-align:center}input:-ms-input-placeholder{color:#c5c8c6;font-style:italic;text-align:center}input::placeholder{color:#c5c8c6;font-style:italic;text-align:center}#configuration{position:fixed;display:none;top:40px;right:15px;background:#373b41;z-index:9999;border:5px solid #282a2e;box-sizing:border-box;box-shadow:2px 2px 2px 0 #202225}#configuration form{padding:10px;display:table;font-family:"Consolas","Menlo","Monaco","Lucida Console","Liberation Mono","DejaVu Sans Mono","Bitstream Vera Sans Mono","Courier New","monospace","serif";font-size:14px;color:#c5c8c6}#configuration p{display:table-row}#configuration label,#configuration input{margin-top:2px;display:table-cell}#configuration label{border:none}#configuration input{margin-left:20px;border:none;width:80px;text-align:right;vertical-align:middle}